package enseirb.myinpulse.service.database;

import enseirb.myinpulse.model.User;
import enseirb.myinpulse.repository.UserRepository;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.server.ResponseStatusException;

import java.util.Optional;

@Service
public class UserService {

    protected static final Logger logger = LogManager.getLogger();

    private final UserRepository userRepository;

    @Autowired
    UserService(UserRepository userRepository) {
        this.userRepository = userRepository;
    }

    public Iterable<User> getAllUsers() {
        return this.userRepository.findAll();
    }

    // TODO
    public User getUserByEmail(String email) {
        Optional<User> opt_user = this.userRepository.findByPrimaryMail(email);

        if (opt_user.isEmpty()) {
            logger.error("getUserByEmail : No user found with email {}", email);
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Cet utilisateur n'existe pas");
        }
        return opt_user.get();
    }

    public Iterable<User> allUsers() {
        return this.userRepository.findAll();
    }

    public User addUser(@RequestBody User user) {
        return this.userRepository.save(user);
    }

    public User updateUser(
            @PathVariable Long id,
            String userSurname,
            String userName,
            String primaryMail,
            String secondaryMail,
            String phoneNumber) {
        Optional<User> user = userRepository.findById(id);
        if (user.isEmpty()) {
            logger.error("updateUser : No user found with id {}", id);
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Cet utilisateur n'existe pas");
        }
        if (userName != null) {
            user.get().setUserName(userName);
        }
        if (userSurname != null) {
            user.get().setUserSurname(userSurname);
        }
        if (primaryMail != null) {
            user.get().setPrimaryMail(primaryMail);
        }
        if (secondaryMail != null) {
            user.get().setSecondaryMail(secondaryMail);
        }
        if (phoneNumber != null) {
            user.get().setPhoneNumber(phoneNumber);
        }
        return this.userRepository.save(user.get());
    }
}
